/*++ BUILD Version: 0001    // Increment this if a change has global effects

Copyright (c) Microsoft Corporation. All rights reserved.

Module Name:

    ntddcdrm.h

Abstract:

    This module contains structures and definitions
    associated with CDROM IOCTls.

--*/

#ifndef NTDDCDRM_H
#define NTDDCDRM_H

//
// NtDeviceIoControlFile IoControlCode values for this device.
//
// Warning:  Remember that the low two bits of the code specify how the
//           buffers are passed to the driver!
//

#define IOCTL_CDROM_BASE                 FILE_DEVICE_CD_ROM // 0x00000002

#define IOCTL_CDROM_UNLOAD_DRIVER        (IOCTL_CDROM_BASE<<16)|(0x0402<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)

//
// CDROM Audio Device Control Functions
//

#define IOCTL_CDROM_READ_TOC         (IOCTL_CDROM_BASE<<16)|(0x0000<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_SEEK_AUDIO_MSF   (IOCTL_CDROM_BASE<<16)|(0x0001<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_STOP_AUDIO       (IOCTL_CDROM_BASE<<16)|(0x0002<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_PAUSE_AUDIO      (IOCTL_CDROM_BASE<<16)|(0x0003<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_RESUME_AUDIO     (IOCTL_CDROM_BASE<<16)|(0x0004<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_GET_VOLUME       (IOCTL_CDROM_BASE<<16)|(0x0005<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_PLAY_AUDIO_MSF   (IOCTL_CDROM_BASE<<16)|(0x0006<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_SET_VOLUME       (IOCTL_CDROM_BASE<<16)|(0x000A<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_READ_Q_CHANNEL   (IOCTL_CDROM_BASE<<16)|(0x000B<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_GET_CONTROL      (IOCTL_CDROM_BASE<<16)|(0x000D<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_GET_LAST_SESSION (IOCTL_CDROM_BASE<<16)|(0x000E<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_RAW_READ         (IOCTL_CDROM_BASE<<16)|(0x000F<<2)|METHOD_OUT_DIRECT|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_DISK_TYPE        (IOCTL_CDROM_BASE<<16)|(0x0010<<)|METHOD_BUFFERED|(FILE_ANY_ACCESS<<14)

#define IOCTL_CDROM_GET_DRIVE_GEOMETRY    (IOCTL_CDROM_BASE<<16)|(0x0013<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_GET_DRIVE_GEOMETRY_EX (IOCTL_CDROM_BASE<<16)|(0x0014<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)

#define IOCTL_CDROM_READ_TOC_EX       (IOCTL_CDROM_BASE<<16)|(0x0015<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_GET_CONFIGURATION (IOCTL_CDROM_BASE<<16)|(0x0016<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)

// end_winioctl

//
// The following device control codes are common for all class drivers.  The
// functions codes defined here must match all of the other class drivers.
//
// Warning: these codes will be replaced in the future with the IOCTL_STORAGE
// codes included below
//

#define IOCTL_CDROM_CHECK_VERIFY    (IOCTL_CDROM_BASE<<16)|(0x0200<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_MEDIA_REMOVAL   (IOCTL_CDROM_BASE<<16)|(0x0201<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_EJECT_MEDIA     (IOCTL_CDROM_BASE<<16)|(0x0202<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_LOAD_MEDIA      (IOCTL_CDROM_BASE<<16)|(0x0203<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_RESERVE         (IOCTL_CDROM_BASE<<16)|(0x0204<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_RELEASE         (IOCTL_CDROM_BASE<<16)|(0x0205<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)
#define IOCTL_CDROM_FIND_NEW_DEVICES (IOCTL_CDROM_BASE<<16)|(0x0206<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)

//
// The following file contains the IOCTL_STORAGE class ioctl definitions
//

#include "ntddstor.h"

//
// The following device control code is for the SIMBAD simulated bad
// sector facility. See SIMBAD.H in this directory for related structures.
//

#define IOCTL_CDROM_SIMBAD        (IOCTL_CDROM_BASE<<16)|(0x1003<<2)|METHOD_BUFFERED|(FILE_READ_ACCESS<<14)

//
// Maximum CD Rom size
//

#define MAXIMUM_NUMBER_TRACKS 100
#define MAXIMUM_CDROM_SIZE 804
#define MINIMUM_CDROM_READ_TOC_EX_SIZE 2  // two bytes min transferred

//
// READ_TOC_EX structure
//
CDROM_READ_TOC_EX STRUCT
	Format    CHAR
	Reserved1 CHAR
	Msf       CHAR
	SessionTrack CHAR
	Reserved2 CHAR
	Reserved3 CHAR
ENDS

#define CDROM_READ_TOC_EX_FORMAT_TOC      0x00
#define CDROM_READ_TOC_EX_FORMAT_SESSION  0x01
#define CDROM_READ_TOC_EX_FORMAT_FULL_TOC 0x02
#define CDROM_READ_TOC_EX_FORMAT_PMA      0x03
#define CDROM_READ_TOC_EX_FORMAT_ATIP     0x04
#define CDROM_READ_TOC_EX_FORMAT_CDTEXT   0x05

//
// CD ROM Table OF Contents (TOC)
// Format 0 - Get table of contents
//

TRACK_DATA STRUCT
	Reserved CHAR
	Control CHAR
	Adr CHAR
	TrackNumber CHAR
	Reserved1 CHAR
	Address CHAR 4 DUP
ENDS

CDROM_TOC STRUCT
	Length CHAR 2 DUP // add two bytes for this field
	FirstTrack CHAR
	LastTrack CHAR
	TrackData TRACK_DATA MAXIMUM_NUMBER_TRACKS DUP
ENDS

#define CDROM_TOC_SIZE sizeof CDROM_TOC

//
// CD ROM Table OF Contents
// Format 1 - Session Information
//

CDROM_TOC_SESSION_DATA STRUCT
	Length CHAR 2 DUP // add two bytes for this field
	FirstCompleteSession CHAR
	LastCompleteSession CHAR
	TrackData TRACK_DATA
ENDS


//
// CD ROM Table OF Contents
// Format 2 - Full TOC
//

CDROM_TOC_FULL_TOC_DATA_BLOCK STRUCT
	SessionNumber CHAR
	Control CHAR
	Adr CHAR
	Reserved1 CHAR
	Point CHAR
	MsfExtra CHAR 3 DUP
	Zero CHAR
	Msf CHAR 3 DUP
ENDS

CDROM_TOC_FULL_TOC_DATA STRUCT
	Length CHAR 2 DUP  // add two bytes for this field
	FirstCompleteSession CHAR
	LastCompleteSession CHAR
	Descriptors CDROM_TOC_FULL_TOC_DATA_BLOCK
ENDS

//
// CD ROM Table OF Contents
// Format 3 - Program Memory Area
//
CDROM_TOC_PMA_DATA STRUCT
	Length CHAR 2 DUP  // add two bytes for this field
	Reserved1 CHAR
	Reserved2 CHAR
	Descriptors CDROM_TOC_FULL_TOC_DATA_BLOCK
ENDS

//
// CD ROM Table OF Contents
// Format 4 - Absolute Time In Pregroove
//

CDROM_TOC_ATIP_DATA_BLOCK STRUCT

	CdrwReferenceSpeed CHAR
	Reserved3 CHAR
	WritePower CHAR
	True1 CHAR
	Reserved4 CHAR
	UnrestrictedUse CHAR
	Reserved5 CHAR
	A3Valid CHAR
	A2Valid CHAR
	A1Valid CHAR
	DiscSubType CHAR
	IsCdrw CHAR
	True2 CHAR
	Reserved7 CHAR
    
	LeadInMsf CHAR 3 DUP
	Reserved8 CHAR
    
	LeadOutMsf CHAR 3 DUP
	Reserved9 CHAR
    
	A1Values CHAR 3 DUP
	Reserved10 CHAR
    
	A2Values CHAR 3 DUP
	Reserved11 CHAR

	A3Values CHAR 3 DUP
	Reserved12 CHAR

ENDS

CDROM_TOC_ATIP_DATA STRUCT
	Length CHAR 2 DUP  // add two bytes for this field
	Reserved1 CHAR
	Reserved2 CHAR
	Descriptors CDROM_TOC_ATIP_DATA_BLOCK
ENDS

//
// CD ROM Table OF Contents
// Format 5 - CD Text Info
//
CDROM_TOC_CD_TEXT_DATA_BLOCK STRUCT
	PackType CHAR
	TrackNumber CHAR
	ExtensionFlag CHAR
	SequenceNumber CHAR
	CharacterPosition CHAR
	BlockNumber CHAR
	Unicode CHAR
	UNION
		Text CHAR 12 DUP
		WText WCHAR 6 DUP
	ENDUNION
	CRC CHAR 2 DUP
ENDS

CDROM_TOC_CD_TEXT_DATA STRUCT
	Length CHAR 2 DUP  // add two bytes for this field
	Reserved1 CHAR
	Reserved2 CHAR
	Descriptors CDROM_TOC_CD_TEXT_DATA_BLOCK

ENDS

//
// These are the types used for PackType field in CDROM_TOC_CD_TEXT_DATA_BLOCK
// and also for requesting specific info from IOCTL_CDROM_READ_CD_TEXT
//
#define CDROM_CD_TEXT_PACK_ALBUM_NAME 0x80
#define CDROM_CD_TEXT_PACK_PERFORMER  0x81
#define CDROM_CD_TEXT_PACK_SONGWRITER 0x82
#define CDROM_CD_TEXT_PACK_COMPOSER   0x83
#define CDROM_CD_TEXT_PACK_ARRANGER   0x84
#define CDROM_CD_TEXT_PACK_MESSAGES   0x85
#define CDROM_CD_TEXT_PACK_DISC_ID    0x86
#define CDROM_CD_TEXT_PACK_GENRE      0x87
#define CDROM_CD_TEXT_PACK_TOC_INFO   0x88
#define CDROM_CD_TEXT_PACK_TOC_INFO2  0x89
// 0x8a - 0x8d are reserved....
#define CDROM_CD_TEXT_PACK_UPC_EAN    0x8e
#define CDROM_CD_TEXT_PACK_SIZE_INFO  0x8f

//
// Play audio starting at MSF and ending at MSF
//

CDROM_PLAY_AUDIO_MSF STRUCT
	StartingM CHAR
	StartingS CHAR
	StartingF CHAR
	EndingM CHAR
	EndingS CHAR
	EndingF CHAR
ENDS

//
// Seek to MSF
//

CDROM_SEEK_AUDIO_MSF STRUCT
	M CHAR
	S CHAR
	F CHAR
ENDS


//
//  Flags for the disk type
//

CDROM_DISK_DATA STRUCT
	DiskData LONG
ENDS

#define CDROM_DISK_AUDIO_TRACK      0x00000001
#define CDROM_DISK_DATA_TRACK       0x00000002

//
// CD ROM Data Mode Codes, used with IOCTL_CDROM_READ_Q_CHANNEL
//

#define IOCTL_CDROM_SUB_Q_CHANNEL    0x00
#define IOCTL_CDROM_CURRENT_POSITION 0x01
#define IOCTL_CDROM_MEDIA_CATALOG    0x02
#define IOCTL_CDROM_TRACK_ISRC       0x03

CDROM_SUB_Q_DATA_FORMAT STRUCT
	Format CHAR
	Track CHAR
ENDS


//
// CD ROM Sub-Q Channel Data Format
//

SUB_Q_HEADER STRUCT
	Reserved CHAR
	AudioStatus CHAR
	DataLength CHAR 2 DUP
ENDS

SUB_Q_CURRENT_POSITION STRUCT
	Header SUB_Q_HEADER
	FormatCode CHAR
	Control CHAR
	ADR CHAR
	TrackNumber CHAR
	IndexNumber CHAR
	AbsoluteAddress CHAR 4 DUP
	TrackRelativeAddress CHAR 4 DUP
ENDS

SUB_Q_MEDIA_CATALOG_NUMBER STRUCT
	Header SUB_Q_HEADER
	FormatCode CHAR
	Reserved CHAR 3 DUP
	Reserved1 CHAR
	Mcval CHAR
	MediaCatalog CHAR 15 DUP
ENDS

SUB_Q_TRACK_ISRC STRUCT
	Header SUB_Q_HEADER
	FormatCode CHAR
	Reserved0 CHAR
	Track CHAR
	Reserved1 CHAR
	Reserved2 CHAR
	Tcval CHAR
	TrackIsrc CHAR 15 DUP
ENDS

SUB_Q_CHANNEL_DATA UNION
	CurrentPosition SUB_Q_CURRENT_POSITION
	MediaCatalog SUB_Q_MEDIA_CATALOG_NUMBER
	TrackIsrc SUB_Q_TRACK_ISRC
ENDS

//
// Audio Status Codes
//

#define AUDIO_STATUS_NOT_SUPPORTED  0x00
#define AUDIO_STATUS_IN_PROGRESS    0x11
#define AUDIO_STATUS_PAUSED         0x12
#define AUDIO_STATUS_PLAY_COMPLETE  0x13
#define AUDIO_STATUS_PLAY_ERROR     0x14
#define AUDIO_STATUS_NO_STATUS      0x15

//
// ADR Sub-channel Q Field
//

#define ADR_NO_MODE_INFORMATION     0x0
#define ADR_ENCODES_CURRENT_POSITION 0x1
#define ADR_ENCODES_MEDIA_CATALOG   0x2
#define ADR_ENCODES_ISRC            0x3

//
// Sub-channel Q Control Bits
//

#define AUDIO_WITH_PREEMPHASIS      0x1
#define DIGITAL_COPY_PERMITTED      0x2
#define AUDIO_DATA_TRACK            0x4
#define TWO_FOUR_CHANNEL_AUDIO      0x8

//
// Get Audio control parameters
//

CDROM_AUDIO_CONTROL STRUCT
	LbaFormat CHAR
	LogicalBlocksPerSecond SHORT
ENDS

//
// Volume control - Volume takes a value between 1 and 0xFF.
// SCSI-II CDROM audio suppports up to 4 audio ports with
// Independent volume control.
//

VOLUME_CONTROL STRUCT
	PortVolume CHAR 4 DUP
ENDS

YellowMode2 = 0
XAForm2 = 1
CDDA = 2

//
// Passed to cdrom to describe the raw read, ie. Mode 2, Form 2, CDDA...
//

_RAW_READ_INFO STRUCT
	DiskOffset LARGE_INTEGER
	SectorCount LONG
	TrackMode DD
ENDS

#endif  // _NTDDCDRM_
